@@ -16,19 +16,19 @@ Promise::XS - Fast promises in Perl
     my $deferred = Promise::XS::deferred();
 
     # Do one of these once you have the result of your operation:
-    $deferred->resolve( 'foo' );
-    $deferred->reject( 'oh no!' );
+    $deferred->resolve( 'foo', 'bar' );
+    $deferred->reject( 'oh', 'no!' );
 
     # Give this to your caller:
     my $promise = $deferred->promise();
 
 The following aggregator functions are exposed:
 
-    # Resolves with an arrayref, one item per promise.
+    # Resolves with a list of arrayrefs, one per promise.
     # Rejects with the results from the first rejected promise.
     my $all_p = Promise::XS::all( $promise1, $promise2, .. );
 
-    # Resolves/rejects with the result from the first
+    # Resolves/rejects with the results from the first
     # resolved or rejected promise.
     my $race_p = Promise::XS::race( $promise3, $promise4, .. );
 
@@ -37,9 +37,9 @@ as C<collect()>.
 
 The following also exist:
 
-    my $pre_resolved_promise = Promise::XS::resolved('already done');
+    my $pre_resolved_promise = Promise::XS::resolved('already', 'done');
 
-    my $pre_rejected_promise = Promise::XS::rejected('badbad');
+    my $pre_rejected_promise = Promise::XS::rejected('it’s', 'bad');
 
 All of C<Promise::XS>’s static functions may be exported at load time,
 e.g., C<use Promise::XS qw(deferred)>.
@@ -48,48 +48,27 @@ e.g., C<use Promise::XS qw(deferred)>.
 
 This module exposes a Promise interface with its major parts
 implemented in XS for speed. It is a fork and refactor of
-L<AnyEvent::XSPromises>, with some significant interface changes.
+L<AnyEvent::XSPromises>. That module’s interface, a “bare-bones”
+subset of that from L<Promises>, is retained.
 
 =head1 STATUS
 
-This module should be fairly stable but is still relatively untested since
-the fork from L<AnyEvent::XSPromises>. Significant breaking changes happened
-from version 0.07 to 0.08. Such changes aren’t expected to be needed again,
-but that’s not guaranteed. Caveat emptor.
+Breaking changes in this interface are unlikely; however, the implementation
+is relatively untested since the fork. Your mileage may vary.
 
-=head1 PROMISE INTERFACE
+=head1 DIFFERENCES FROM ECMASCRIPT PROMISES
 
-The core functionality derives from the
-L<Promises/A+ specification|https://promisesaplus.com/> standard. (See
-L</EVENT LOOPS> below for one important difference.) Its C<finally()>
-implementation L<derives from ECMAScript promises|https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise/finally>.
-
-=head2 Promise callbacks: list vs. scalar context
-
-Most Perl promise libraries allow promises to resolve or reject with multiple
-values. This is eminently “perlish”, but what do we do if a plural return
-includes a promise? Neither Promises/A+ nor ECMAScript’s promise standard
-describes how to handle this scenario, and there’s no “obvious” solution
-otherwise. We could simply ignore the “extra” inputs, but what if one of those
-“extras” is itself a promise? What if there’s only one promise, but it’s not
-the first item returned?
-
-All of these scenarios allow for subtle bugs to arise. To avoid that,
-this library executes all callbacks in scalar context. Besides avoiding the
-“problem” states described above, this also matches both Promises/A+ and
-ECMAScript standards. The divergence from preexisting Perl promise libraries
-like L<Mojo::Promise>, L<Promises>, and L<AnyEvent::XSPromises>, is
-regrettable but seems a “lesser evil” overall.
-
-=head2 Additional notes
+This library is built for compatibility with pre-existing Perl promise
+libraries. It thus exhibits some salient differences from how
+ECMAScript promises work:
 
 =over
 
 =item * Neither the C<resolve()> method of deferred objects
 nor the C<resolved()> convenience function define behavior when given
-a promise object. Don’t do it.
+a promise object.
 
-=item * The C<all()> and C<race()> functions accept a list,
+=item * The C<all()> and C<race()> functions accept a list of promises,
 not a “scalar-array-thing” (ECMAScript “arrays” being what in Perl we
 call “array references”). So whereas in ECMAScript you do:
 
@@ -99,11 +78,50 @@ call “array references”). So whereas in ECMAScript you do:
 
     Promise::XS::all( $promise1, $promise2 );
 
+=item * Promise resolutions and rejections may contain multiple values.
+(But see L</AVOID MULTIPLES> below.)
+
 =back
 
 See L<Promise::ES6> for an interface that imitates ECMAScript promises
 more closely.
 
+=head1 AVOID MULTIPLES
+
+For compatibility with preexisting Perl promise libraries, Promise::XS
+allows a promise to resolve or reject with multiple values. This behavior,
+while eminently “perlish”, allows for some weird cases where the relevant
+standards don’t apply: for example, what happens if multiple promises are
+returned from a promise callback? Or even just a single promise plus extra
+returns?
+
+Promise::XS tries to help you catch such cases by throwing a warning
+if multiple return values from a callback contain a promise as the
+first member. For best results, though—and consistency with promise
+implementations outside Perl—resolve/reject all promises with I<single>
+values.
+
+=head1 DIFFERENCES FROM L<Promises> ET AL.
+
+This module implements ECMAScript’s C<finally()> interface, which differs
+from that in some other Perl promise implementations.
+
+Given the following …
+
+    my $new = $p->finally( $callback );
+
+=over
+
+=item * C<$callback> receives I<no> arguments.
+
+=item * If C<$callback> returns anything but a single, rejected promise,
+C<$new> has the same status as C<$p>.
+
+=item * If C<$callback> throws or returns a single, rejected promise,
+C<$new> is rejected with C<$callback>’s exception.
+
+=back
+
 =head1 EVENT LOOPS
 
 By default this library uses no event loop. This is a perfectly usable
@@ -213,80 +231,17 @@ sub rejected {
 
 #----------------------------------------------------------------------
 # Aggregator functions
-
-# Lifted from AnyEvent::XSPromises
 sub all {
-    my $remaining= 0+@_;
-    my @values;
-    my $failed= 0;
-    my $then_what= deferred();
-    my $pending= 1;
-    my $i= 0;
+    return Promise::XS::Promise->all(@_);
+}
 
-    my $reject_now = sub {
-        if (!$failed++) {
-            $pending= 0;
-            $then_what->reject(@_);
-        }
-    };
-
-    for my $p (@_) {
-        my $i = $i++;
-
-        $p->then(
-            sub {
-                $values[$i]= $_[0];
-                if ((--$remaining) == 0) {
-                    $pending= 0;
-                    $then_what->resolve(\@values);
-                }
-            },
-            $reject_now,
-        );
-    }
-    if (!$remaining && $pending) {
-        $then_what->resolve(@values);
-    }
-    return $then_what->promise;
+sub race {
+    return Promise::XS::Promise->race(@_);
 }
 
 # Compatibility with other promise interfaces.
 *collect = *all;
 
-# Lifted from Promise::ES6
-sub race {
-
-    my $deferred = deferred();
-
-    my $is_done;
-
-    my $on_resolve_cr = sub {
-        return if $is_done;
-        $is_done = 1;
-
-        $deferred->resolve(@_);
-
-        # Proactively eliminate references:
-        undef $deferred;
-    };
-
-    my $on_reject_cr = sub {
-        return if $is_done;
-        $is_done = 1;
-
-        $deferred->reject(@_);
-
-        # Proactively eliminate references:
-        undef $deferred;
-    };
-
-    for my $given_promise (@_) {
-        $given_promise->then($on_resolve_cr, $on_reject_cr);
-    }
-
-    return $deferred->promise();
-}
-
 #----------------------------------------------------------------------
 
 =head1 SEE ALSO
